# frozen_string_literal: true

module Nanoc
  module Core
    module ProcessingActions
    end
  end
end

require 'nanoc/core/processing_actions/filter'
require 'nanoc/core/processing_actions/layout'
require 'nanoc/core/processing_actions/snapshot'
