# frozen_string_literal: true

describe 'GH-951', :site, :stdio do
  before do
    File.write('content/foo.md', 'Foo!')

    File.open('nanoc.yaml', 'w') do |io|
      io << 'string_pattern_type: legacy' << "\n"
    end

    File.write('Rules', <<EOS)
  passthrough '/foo.md'
EOS
  end

  it 'copies foo.md' do
    Nanoc::CLI.run(['compile'])

    expect(File.file?('output/foo.md')).to be(true)
  end
end
