# frozen_string_literal: true

describe 'GH-833', :site, :stdio do
  before do
    File.write('content/foo.md', 'stuff')
    File.write('Rules', <<EOS)
  compile '/**/*' do
    write item.identifier.without_ext + '.txt'
  end
EOS
  end

  it 'runs show-data without crashing' do # rubocop:disable RSpec/NoExpectationExample
    Nanoc::CLI.run(['show-data'])
  end
end
