# frozen_string_literal: true

# @api private
module Nanoc::Filters
end

require_relative 'filters/asciidoc'
require_relative 'filters/asciidoctor'
require_relative 'filters/bluecloth'
require_relative 'filters/colorize_syntax'
require_relative 'filters/coffeescript'
require_relative 'filters/erb'
require_relative 'filters/erubi'
require_relative 'filters/erubis'
require_relative 'filters/haml'
require_relative 'filters/handlebars'
require_relative 'filters/kramdown'
require_relative 'filters/less'
require_relative 'filters/markaby'
require_relative 'filters/maruku'
require_relative 'filters/mustache'
require_relative 'filters/pandoc'
require_relative 'filters/rainpress'
require_relative 'filters/rdiscount'
require_relative 'filters/rdoc'
require_relative 'filters/redcarpet'
require_relative 'filters/redcloth'
require_relative 'filters/relativize_paths'
require_relative 'filters/rubypants'
require_relative 'filters/sass'
require_relative 'filters/slim'
require_relative 'filters/typogruby'
require_relative 'filters/terser'
require_relative 'filters/xsl'
require_relative 'filters/yui_compressor'
