# frozen_string_literal: true

describe 'GH-1428', :site, :stdio do
  before do
    FileUtils.mkdir_p('layouts')
    File.write('layouts/default.erb', 'layout stuff')

    File.write('Rules', <<~EOS)
      ignore '/*'
      layout '/*', :erb
    EOS
  end

  example do # rubocop:disable RSpec/NoExpectationExample
    Nanoc::CLI.run([])
    Nanoc::CLI.run(['show-data'])
  end
end
